﻿using System;
using System.Collections.Generic;
using System.Text;
using SuperMap.Desktop;
using SuperMap.Data;
using SuperMap.UI;
using SuperMap.Mapping;
using System.Windows.Forms;
using System.Drawing;

namespace SuperMap.Desktop.OverViewMap
{
    //Function bound to the Display Navigation Map button


    //A class inheriting from the CtrlAction class or implementing the ICtrlAction interface
    class _CtrlActionOverViewMap : CtrlAction
    {
        //Use base class (CtrlAction) constructor with parameteres

        public _CtrlActionOverViewMap(IBaseItem caller)
            : base(caller)
        {

        }
        /// <summary>
        /// Navigation map control

        /// </summary>
        private MapControl _LocationMapCtl = null;
        /// <summary>
        /// Workspace control
        /// </summary>
        private Workspace _WorkspaceCtl = null;
        /// <summary>
        /// Rectangle box, defines the visible extent of the map
        /// </summary>
        private GeoRectangle objRectangle = null;
        /// <summary>
        /// Return navigation map control
        /// </summary>
        public MapControl LocationMapCtl
        {
            get { return this._LocationMapCtl; }
        }
        /// <summary>
        /// Return workspace control
        /// </summary>
        public Workspace WorkspaceCtl
        {
            get { return this._WorkspaceCtl; }
        }

        public override void Run()
        {
            try
            {
                //Create a new window
                Form frmLocation = new Form();
                frmLocation.Text = "Navigation Map";
                frmLocation.StartPosition = FormStartPosition.Manual;
                frmLocation.FormBorderStyle = FormBorderStyle.SizableToolWindow;
                frmLocation.TopMost = true;
                //Create a new map control
                _LocationMapCtl = new MapControl();
                LocationMapCtl.Dock = DockStyle.Fill;
                frmLocation.Controls.Add(LocationMapCtl);
                //Display the window
                frmLocation.Show();

                //Draw map in the window

                //Get layers of current active map window
                IFormMap mapWindow = Application.ActiveForm as IFormMap;
                
                LocationMapCtl.Map.Workspace = mapWindow.MapControl.Map.Workspace;
                bool bOpen = LocationMapCtl.Map.FromXML(mapWindow.MapControl.Map.ToXML());
                LocationMapCtl.Map.ViewEntire();
              

                if (bOpen)
                {
                    SuperMap.Desktop.Application.ActiveApplication.Output.Output("Succeeded to add navigation layer");
                    //Delegation event
                    mapWindow.MapControl.Map.Drawn += new MapDrawnEventHandler(MapControl_DrawnHandler);
                    mapWindow.MapControl.Focus();
                    LocationMapCtl.MouseDown += new MouseEventHandler(LocationMapCtl_MouseDownHandler);
                }
                else
                {
                    throw new Exception("Failed to add navigation layer");
                }

            }
            catch (Exception ex)
            {
                SuperMap.Desktop.Application.ActiveApplication.Output.Output(ex.Message);
            }
        }

        private void MapControl_DrawnHandler(object sender, MapDrawnEventArgs e)
        {
            //Get layers of current active map window
            IFormMap mapWindow = Application.ActiveForm as IFormMap;
            //Create a rectangle box
            objRectangle = new GeoRectangle();
            objRectangle.Center = mapWindow.MapControl.Map.Center;
            objRectangle.Width = mapWindow.MapControl.Map.ViewBounds.Width;
            objRectangle.Height = mapWindow.MapControl.Map.ViewBounds.Height;
            //Set style for the rectangle box

            GeoStyle style = new GeoStyle();
            style.LineColor = Color.Blue;
            style.LineWidth = 0.5;
            style.LineSymbolID = 6;
            style.FillBackOpaque = false;
            style.FillOpaqueRate = 0;
            objRectangle.Style = style;
            
            //Add to tracking layer

            if ( !LocationMapCtl.IsDisposed)
            {
                TrackingLayer trackLayer = LocationMapCtl.Map.TrackingLayer;
                //Empty tracking layer

                trackLayer.Clear();
                //Add object
                trackLayer.Add(objRectangle as Geometry, "Navigation Map");
                LocationMapCtl.Map.RefreshTrackingLayer();
                LocationMapCtl.Refresh();
            }
            
        }

        private void LocationMapCtl_MouseDownHandler(object sender, MouseEventArgs e)
        {
            //Add to tracking layer again
            TrackingLayer trackLayer = LocationMapCtl.Map.TrackingLayer;
            //Empty tracking layer

            trackLayer.Clear();
            //Move object
            Point pntCenter = new Point(e.X, e.Y);
            objRectangle.Center = LocationMapCtl.Map.PixelToMap(pntCenter);
            //Add object
            trackLayer.Add(objRectangle as Geometry, "Navigation Map");
            //Get layers of current active map window
            IFormMap mapWindow = Application.ActiveForm as IFormMap;
            mapWindow.MapControl.Map.ViewBounds = objRectangle.Bounds;

            //Refresh
            mapWindow.MapControl.Map.Refresh();
            mapWindow.MapControl.Refresh();
        }
    }
}
